#include "WProgram.h"
/*
 * "ArduinomeFirmware" - Arduino Based Monome Clone by Owen Vallis & Jordan Hochenbaum 06/16/2008
 * Revised 06/26/2008
 * Revised 07/20/2008 by Ben Southall
 *
 * --------------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * --------------------------------------------------------------------------
 *
 * This code is largely a direct translation of the 40h protocol designed by Brian Crabtree
 * & Joe Lake, and coded by Joe Lake.  We would like to express our utmost thanks for making
 * the monome & its inner-workings available and open source.  We would also like to extend
 * our thanks to all those who have helped us get this far in successfully making an arduino-
 * based monome clone, including but not limited to Brad Hill for his awesome arduino shield, 
 * and Melka for showing how to pack and unpack the serial data.
 *
 * Additional comment and attribution 7/20/2008 : 
 * The changes on 7/20/2008 were inspired by, and at least in some part, directly
 * copied from the 'Octinct' code written by Jonathan M Guberman (upwardnotnorthward.com).  
 * In particular, the use of timer 2 to drive an interrupt routine to read the serial port was
 * picked up from the Octinct project, and also, setting the baud rate to 57600 seemed to be 
 * critical for getting everything to work reliably.  My thanks to Jonathan for sharing his
 * software, and for figuring out how to keep that serial buffer in check.
 *
 * Please DO NOT email monome with technical questions and/or help regarding this code or clone.  
 * They are in NO WAY responsible or affiliated with this project other than they were our inspiration 
 * and we used many of their methods and pulled from their code.
 * 
 * Additionally, while we are availble and willing to help as much as possible, we too CANNOT be held
 * responsible for anything you do with this code.  Please feel free to report any bugs, suggestions 
 * or improvements to us as they are all welcome.  Again, we cannot be held responsible for any damages 
 * or harm caused by the use or misuse of this code or our instructions.  Thank you for understanding.
 * --------------------------------------------------------------------------
 *
 * Links:
 * http://bricktable.wordpress.com - Our website - Click "Arduino Monome Project" on the Navigation Menu on the right.
 * www.monome.org - the "original" monome and our inspiration
 * www.flickr.com/photos/unsped/2283428540/in/photostream/
 * http://play-collective.net/blog/archives/category/arduinomonomergb
 *
 */



/*
Ok, so I'm gonna but a few defines in here
 
 */

//these are to set the serial protocol driver speed

#define OSX 1
#define WIN 0
#define LINUX 0


/* Here is where you choose which display driver to use*/

#define DISPLAY_DRIVER_UNSPED 0
#define DISPLAY_DRIVER_LARSBY 1
#define DISPLAY_DRIVER_EMPTY 0

/* Here is where you choose which display driver to use */

#define BUTTON_DRIVER_UNSPED 0
#define BUTTON_DRIVER_LARSBY 1
#define BUTTON_DRIVER_EMPTY 0




//------------------------------------------------------------------------------------------
// Button typedefs

void init_buttons_driver();
void toggleAllButtons();
void populate_buttons ();
byte get_button_states();
void pulse_pin(int pin_number);
byte clear_lsb(byte byte_to_clear);
void populate_buttons ();
void init_buttons_driver();
void buttonInit(void);
void buttonCheck(byte row, byte index);
void check_and_send_buttons(void);
void displayInit();
void displayWhoosh();
void ChangeIntensity(int inval);
void ChangeDisplayTest(byte inval);
void ShutdownMode(byte inval);
void updateLED();
void clearAllLights();
void lightAllLed();
void toggleLed(int x, int y);
void setLed(int x, int y);
void clearLed(int x, int y);
void displayInit();
void displayWhoosh();
void putByte(byte data);
void maxSingle( byte dig, byte seg);
void ChangeIntensity(int inval);
void ChangeDisplayTest(byte inval);
void ShutdownMode(byte inval);
void updateLED();
void clearAllLights();
void initInterrupt();
void whoosh(void);
void setup ();
void loop ();
void goProtocol();
byte button_current[8];
byte button_last[8];
byte button_state[8];
byte button_event[8];

void init_buttons_driver();

//function to read the state and populate the button array's
void populate_buttons();


//------------------------------------------------------------------------------------------
// Display typedefs
byte ledmem[8];                         // memory for LEDs



//function to display pretty patterns on screen to show that the arduino is inited properrly
void displayWhoosh();

//initiates variables and such
void displayInit();


//changes the intensity of the screen
void ChangeIntensity(int inval);

//init's a display test function
void ChangeDisplayTest(byte inval);

//sets the sceen into shutdownmode 
void ShutdownMode(byte inval);

//main loop update thingy
void updateLED();
//utility function to wipe the screen
void clearAllLights();






/* changing to defines where approperiate made size go from 4712 to 4350 bytes with unsped drivers *//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#if BUTTON_DRIVER_LARSBY


#define pin_button_data  10
#define pin_button_clock  9
#define pin_button_apl  8

#define pin_button_latch  pin_button_apl

void init_buttons_driver()
{
  pinMode(pin_button_latch, OUTPUT);
  pinMode(pin_button_clock, OUTPUT); 
  pinMode(pin_button_data, INPUT);
}

void toggleAllButtons() //will swap all the bytes, the hardware I use gives 1 as default
{

   for (int I = 0; I < 8; I++)
   for (int y  = 0; y < 8; y++)
      button_current[I]  ^= (1 << (7-y));
}


//function to read the state and populate the button array's
void populate_buttons () {
  
    for (int i = 0; i < 8; i++)
      button_last [i] = button_current [i]; 


   pulse_pin(pin_button_latch); // sample the button states

 for (int I = 0; I < 8; I++)
  button_current[I]= get_button_states();
  toggleAllButtons();
}


byte get_button_states(){
  byte my_bit;
  byte button_byte;

  button_byte=0; // clear the byte ready for new data
  for (int n=0; n<8; n++){
    button_byte = button_byte << 1 ; // shift bits to the left, making space to capture the state of the next button
    button_byte=clear_lsb(button_byte); // make sure the new lsb is zero, to avoid surprises
    my_bit = digitalRead(pin_button_data); // store the current Q7 value from the button register in my_bit 
    button_byte = button_byte | my_bit; // OR th my_input with the new bit to add it to the lsb 
    pulse_pin(pin_button_clock); // cue the next bit slot for reading
  }
  return(button_byte);
}


// Set a pin to low, then high
void pulse_pin(int pin_number){
  digitalWrite(pin_number,LOW);
  //delayMicroseconds(20);
  digitalWrite(pin_number,HIGH);
}

// Clear the least significant bit
byte clear_lsb(byte byte_to_clear){
  return(byte_to_clear & 0xfe);  // AND the byte with 11111110 to be sure that the LSB is zero
}

 

#endif


#if BUTTON_DRIVER_UNSPED

//164 (from serial to IC)               // set which pins to use
// load data from computer to 164
#define dataPin  3
// pulse to load bits (one bit for each clock pulse)
#define clockPin  2

//165 (from IC to serial)
// toggling this tells the 165 to read the byte into its memory for reading
#define inloadPin   8                     
// pull the data out of the 165 bit by bit 
#define indataPin   9                     
// pulse to pull bits (one bit for each clock pulse)
#define inclockPin   7                     

byte t = 0;
int id = 0;

//164 (from serial to IC)               // set which pins to use
//  dataPin = 3 = PORTD, bit 3
// clockPin = 2 = PORTD, bit 2
#define dataPinMaskHigh  (1 << 3)
#define dataPinMaskLow   (~dataPinMaskHigh)


#define clockPinMaskHigh ( 1 << 2)
#define clockPinMaskLow (~clockPinMaskHigh)

//165 (from IC to serial)
//  inloadPin = 8 = PORTB, bit 0
// indataPin = 9 = PORTB, bit 1
// inclockPin = 7 = PORTD, bit 7
#define inloadPinMaskHigh  1
#define inloadPinMaskLow ( ~inloadPinMaskHigh)


#define indataPinMask (1 << 1)

#define inclockPinMaskHigh  (1 << 7)
#define inclockPinMaskLow  (~inclockPinMaskHigh)


/* this function populates the buffers with the current state of the buttons */

void populate_buttons () {


  int i = 0;

  PORTD &= dataPinMaskLow; // digitalWrite(dataPin, LOW);
  //164
  for(i = 0; i < 8; i++){
    PORTD |= clockPinMaskHigh; // digitalWrite(clockPin, HIGH);
    PORTD &= clockPinMaskLow;  //digitalWrite(clockPin, LOW);
    PORTD |= dataPinMaskHigh;  //digitalWrite(dataPin, HIGH);

    volatile int a = 0; 

    while (a < 15) 
    { 
      a++; 
    } 

    button_last [i] = button_current [i]; 

    //165
    PORTB &= inloadPinMaskLow; // digitalWrite(inloadPin, LOW);
    PORTB |= inloadPinMaskHigh;// digitalWrite(inloadPin, HIGH);

    for(id = 0; id < 8; id++) {

      t = (PINB & indataPinMask) >> 1;//t = digitalRead(indataPin);
      t = (t == 0);
      if(t){
        button_current [i] |= (1 << id);
      }
      else{
        button_current [i] &= ~(1 << id);
      }

      PORTD |= inclockPinMaskHigh; // digitalWrite(inclockPin, HIGH);
      PORTD &= inclockPinMaskLow;  // digitalWrite(inclockPin, LOW);
    }
  } 

}

void init_buttons_driver()
{

}


#endif

#define kButtonUpDefaultDebounceCount    12
#define kButtonNewEvent    1
#define kButtonNoEvent     0

byte button_debounce_count[8][8];




void buttonInit(void){
  byte i;
  for (i = 0; i < 8; i++) {
    button_current[i] = 0x00;
    button_last[i] = 0x00;
    button_state[i] = 0x00;
    button_event[i] = 0x00;
  }

  init_buttons_driver();
}


void buttonCheck(byte row, byte index)
{
  if (((button_current[row] ^ button_last[row]) & (1 << index)) &&   // if the current physical button state is different from the
  ((button_current[row] ^ button_state[row]) & (1 << index))) {  // last physical button state AND the current debounced state

    if (button_current[row] & (1 << index)) {                      // if the current physical button state is depressed
      button_event[row] = kButtonNewEvent << index;              // queue up a new button event immediately
      button_state[row] |= (1 << index);                         // and set the debounced state to down.
    }
    else{
      button_debounce_count[row][index] = kButtonUpDefaultDebounceCount;
    }  // otherwise the button was previously depressed and now
    // has been released so we set our debounce counter.
  }
  else if (((button_current[row] ^ button_last[row]) & (1 << index)) == 0 &&  // if the current physical button state is the same as
  (button_current[row] ^ button_state[row]) & (1 << index)) {        // the last physical button state but the current physical
    // button state is different from the current debounce 
    // state...

    if (button_debounce_count[row][index] > 0 && --button_debounce_count[row][index] == 0) {  // if the the debounce counter has
      // been decremented to 0 (meaning the
      // the button has been up for 
      // kButtonUpDefaultDebounceCount 
      // iterations///

      button_event[row] = kButtonNewEvent << index;    // queue up a button state change event

      if (button_current[row] & (1 << index)){          // and toggle the buttons debounce state.
        button_state[row] |= (1 << index);
      }
      else{
        button_state[row] &= ~(1 << index);
      }
    }
  }
}



void check_and_send_buttons(void)
{
  int b = 0;

  for(int i = 0; i < 8; i++)
  {

    for(int id = 0; id < 8; id++) 
    {

      buttonCheck(i, id);

      if (button_event[i] & (1 << id)) 
      {
        button_event[i] &= ~(1 << id);
        if(button_state[i] & (1 << id))
        {
          b = 1;
        }
        else
        {
          b = 0;
        }
      //  toggleLed(7,0); den h\u00e4r fungerar ju!
        Serial.print((0 << 4) | (b & 15), BYTE);
        Serial.print((id << 4) | (i & 15), BYTE);
 //       Serial.print((0 << 4) | (b & 15), DEC);
   //     Serial.print(" ");
     //   Serial.println((id << 4) | (i & 15), DEC);
      }
    }
  }
}


#if DISPLAY_DRIVER_LARSBY


//Pin connected to ST_CP of 74HC595
#define latchPin  11
//Pin connected to SH_CP of 74HC595
#define  clockPin  12
////Pin connected to DS of 74HC595
#define  dataPin  13

/* This is correct, however there seems to be some kind of bug in my shiftout so I've flipped the pins here instead :) DIRTY FIX
 int ledCol[8] = {
 7,6,5,4,
 //analog
 14,15,16,17  };
 */
int ledCol[8] = {  
  17,7,6,5,4,14,15,16  }; //did I mention this was a dirty fix?




void displayInit()
{
  for (int i = 0; i < 8; i++)
    pinMode(ledCol[i], OUTPUT);

  pinMode(latchPin, OUTPUT);
  pinMode(clockPin, OUTPUT); 
  pinMode(dataPin, OUTPUT);



}

//function to display pretty patterns on screen to show that the arduino is inited properrly
void displayWhoosh() //have not found a good way to do this yet, since I'm not in the thread yet. well well.
{
  //lightAllLed();
}

//changes the intensity of the screen
void ChangeIntensity(int inval)
{
  //this seems like extravagant stuff to me
}

//init's a display test function
void ChangeDisplayTest(byte inval)
{

  //whoho, I dont have that
}

//sets the sceen into shutdownmode 
void ShutdownMode(byte inval)
{

  //nothing here for that
}

//main loop update thingy
void updateLED()
{

  for (int I = 0; I < 8; I++)
  {
    digitalWrite(ledCol[I],LOW); //turning our current column on first on!

    digitalWrite(latchPin, LOW);
    shiftOut(dataPin, clockPin, LSBFIRST,ledmem[I]);   

    //return the latch pin high to signal chip that it 
    //no longer needs to listen for information
    digitalWrite(latchPin, HIGH);
    digitalWrite(ledCol[I],HIGH); //turning the current column it off!

  }



}

//utility function to wipe the screen

void clearAllLights()
{
  for (int I = 0; I < 8; I++)
    ledmem[I]=0;
}

void lightAllLed()
{
  for (int I = 0; I < 8; I++)
    ledmem[I]=255;
}

void toggleLed(int x, int y)
{  
 ledmem[x]  ^= (1 << (7-y));
}

void setLed(int x, int y)
{
 ledmem[x] |= (1 << (7-y));
}


void clearLed(int x, int y)
{
ledmem[x]  &= ~(1 << (7-y));
}


#endif

#if DISPLAY_DRIVER_UNSPED

// NOTE by defining SPI_MAX as 1, we will activate
// the SPI version of the code that clocks the LED
// data to the max chip.  This will require that
// you also make the following pin connections
// from the arduino to the MAX7219 chip:
// Arduino pin 10 : to max 'load'
// Arduino pin 11 : to max 'din'
// Arduino pin 13 : to max 'clock'
#define SPI_MAX 0

//Max7219 (drives LEDS)
// loads in a 1 (high) or a 0 (low) on clock pulse                        
#define dataIn  4
// pulse to load bits (one bit for each clock pulse)
#define clock  6                         
// outputs all data from the IC to the LEDS
#define load  5                           

int ledstate = 0;
int ledx = 0;
int ledy = 0;
int inputrow = 0;
// change this variable to set how many MAX7219's you'll use
#define maxInUse  1

// define max7219 registers (read the tech doc for explaination)
#define max7219_reg_noop         0x00    
#define max7219_reg_digit0       0x01
#define max7219_reg_digit1       0x02
#define max7219_reg_digit2       0x03
#define max7219_reg_digit3       0x04
#define max7219_reg_digit4       0x05
#define max7219_reg_digit5       0x06
#define max7219_reg_digit6       0x07
#define max7219_reg_digit7       0x08
#define max7219_reg_decodeMode   0x09
#define max7219_reg_intensity    0x0a
#define max7219_reg_scanLimit    0x0b
#define max7219_reg_shutdown     0x0c
#define max7219_reg_displayTest  0x0f

#define DataMaskHigh  (1 << 4)
#define DataMaskLow  (~DataMaskHigh)

#define ClockMaskHigh ( 1<<6)
#define ClockMaskLow  (~ClockMaskHigh)

#define LoadMaskHigh  (1 << 5)
#define LoadMaskLow  (~LoadMaskHigh)

void displayInit()
{

   pinMode(dataPin,    OUTPUT);
  pinMode(clockPin,   OUTPUT);
  pinMode(indataPin,  INPUT);
  pinMode(inclockPin, OUTPUT);
  pinMode(inloadPin,  OUTPUT);

#if SPI_MAX
  SPCR = (1 << SPE) | (1 << MSTR);
  SPSR = (SPI2X);

  DDRB |= (1 << PB2)|(1 << PB3)|(1 << PB5);
#else
  pinMode(dataIn, OUTPUT);
  pinMode(clock,  OUTPUT);
  pinMode(load,   OUTPUT);  
#endif

  
   //initiation of the max 7219
  maxSingle(max7219_reg_scanLimit, 0x07);      
  maxSingle(max7219_reg_intensity,0x0F);
  maxSingle(max7219_reg_shutdown, 0x01);           // not in shutdown mode
  maxSingle(max7219_reg_displayTest, 0x00);        // empty registers, turn all LEDs off h

  for (int e=1; e<=8; e++){
    maxSingle(e,0);
    ledmem[e-1]=0;
  }

}

void displayWhoosh()
{
 for (int j = 0; j < 9; j++)
  {
    for (int i = 0; i < 8; i++)
    {
      maxSingle(i+1,1<<j);
    }
    delay(125);
  }
}




void putByte(byte data) {
  byte i = 8;
  byte mask;
  while(i > 0) {
    mask = 0x01 << (i - 1);      // get bitmask


    if (data & mask){            // choose bit 0 or 1
      PORTD |= DataMaskHigh; //      digitalWrite(dataIn, HIGH);// send 1
    }
    else{                        // else 
      PORTD &= DataMaskLow;   //  digitalWrite(dataIn, LOW); // send 0
    }


    PORTD &= ClockMaskLow; //    digitalWrite( clock, LOW);   // "tick" prepeare for bit input
    PORTD |= ClockMaskHigh;//    digitalWrite(clock, HIGH);   // "tock" input bit

    --i;                         // move to lesser bit
  }
}



void maxSingle( byte dig, byte seg) {    
  //maxSingle is the "easy"  function to use for a single max7219
  //dig is the row call, and seg is the column call dig and seg refer to pin names from tech doc

#if SPI_MAX

  PORTB &= ~(1 << PB2);
  SPDR = dig;
  while (!(SPSR & (1 << SPIF)));
  SPDR = seg;
  while (!(SPSR & (1 << SPIF)));
  PORTB |= (1 << PB2);

#else // SPI_MAX is 0

  PORTD &= LoadMaskLow;  //  digitalWrite(load, LOW);            // begin     

  putByte(dig);                       // specify register
  putByte(seg);                       //((data & 0x01) * 256) + data >> 1); // put data   

  PORTD |= LoadMaskHigh; //  digitalWrite(load,HIGH); 

#endif // end #if SPI_MAX
}

void ChangeIntensity(int inval)
{
  maxSingle(max7219_reg_intensity, inval & 15);
 
}

void ChangeDisplayTest(byte inval)
{
  maxSingle(max7219_reg_displayTest, inval & 15);
 
}

void ShutdownMode(byte inval)
{
   maxSingle(max7219_reg_shutdown, inval & 15); 

}

void updateLED()
{

// send the LED states to the LED matrix.
  for (int i = 0; i < 8; i++)
  {
    maxSingle(i+1,ledmem[i]);
  }
}
  
 void clearAllLights()
 {
 
     // make sure to turn off the lights.
  for (int i = 0; i < 8; i++)
  {
    maxSingle(i+1,0);
  }
 }
 
 #endif



void initInterrupt()

{

  // Set up 8-bit counter 2, output compare switched off,
  // normal waveform generation (whatever that might mean)
  TCCR2A = 0;
  // set counter to be clocked at 16Mhz/8 = 2Mhz
  TCCR2B = 1<<CS21;

  // set the interrupt mask so that we get an interrupt
  // on timer 2 overflow, i.e. after 256 clock cycles.
  // The timer 2 interrupt routine will execute every
  // 128 uS.
  TIMSK2 = 1<<TOIE2;

  // NOTE: In my efforts to try to get this
  // code to work reliably at 115200 baud
  // I went about disabling unused timers that
  // are set up by the Arduino framework.
  // The framework sets up both timer 2 and 
  // timer 1.  We use timer 2 to drive the
  // serial reading interrupt, so we can only
  // disable timer1 and its interrupt.

  // REALLY IMPORTANT NOTE - IF YOU WANT TO USE
  // ANALOGWRITE

  // Disabling timer 1 will switch off the timer
  // used for PWM, which underlies analogWrite.
  // If you want to use analogWrite, remove
  // the lines below.

  // DISABLE PWM COUNTER
  TIMSK0 &= ~(1 << TOIE0);

  TCCR1B &= ~(1 << CS12);
  TCCR1B &= ~(1 << CS11);
  TCCR1B &= ~(1 << CS10);  
  // END DISABLE PWM COUNTER

  // Also, disable the interrupts on timer 0
  // REALLY IMPORTANT NOTE IF YOU WANT TO USE
  // DELAY
  // remove this line, and also look at
  // 'whoosh', which enables and then disables
  // the intterupt on timer 0.  You'll want
  // to get rid of those lines too.
  TIMSK0 &= ~(1 << TOIE0);


}

ISR(TIMER2_OVF_vect) {


 // goProtocol();


}



void whoosh(void)
{
  // reactivate overflow interrupt for
  // timer 1 - it's needed by delay(...)
  TIMSK0 |= (1 << TOIE0); 

  displayWhoosh();

  // and switch the interrupt off.
  TIMSK0 &= ~(1 << TOIE0);
}


void setup () {

  int e = 0;                              // used to initialize the rows to 0



//#if OSX
  Serial.begin(9600);
/*#elif WIN
  Serial.begin(57600);
#elif LINUX
  Serial.begin(57600);
#endif 
*/

  buttonInit();  
  displayInit();
//  initInterrupt();

  // pretty pattern to assure me that the firmware's
  // uploaded properly.
  whoosh();

  clearAllLights();

}  


void loop () {

  // check for button presses
  populate_buttons();
  


 updateLED();
 
  check_and_send_buttons();  
 goProtocol();


}

byte WaitingForAddress = 1;


byte byte0 = 0x00;                      // these are used to hold the serial data
byte byte1 = 0x00;
byte address =  0x00;                   // garbage byte to hold address of function
byte state = 0x00;                      // garbage byte to hold state value
byte x = 0x00;                          // garbage byte to hold x position
byte y = 0x00;                          // garbage byte to hold y position
byte z = 0x00;

byte firstRun = 1;


void goProtocol() {
  // first up: enable interrupts to keep the serial
  // class responsive
  sei(); 

  do 
  {
    if (Serial.available())
    {
      setLed(0,0);
      if (WaitingForAddress == 1)
      {
        byte0 = Serial.read();

        address = byte0 >> 4;
        WaitingForAddress = 0;
      } // end if (WaitingForAddress == 1);

      if (Serial.available())
      {
      setLed(0,1);
        WaitingForAddress = 1;
        byte1 = Serial.read();
  
        switch(address)
        {
      setLed(0,2);

          case 2: //input LED
    
          state = byte0 & 15;
          x = byte1 >> 4;
          y = byte1 & 15;

          if (state == 0){
            ledmem[y] &= ~( 1 << x); 
          }
          else {
            ledmem[y] |=  ( 1 << x);
          }
          break;
        case 3: //led_intensity
      ChangeIntensity(byte1 & 15);   //calling the driver function instead       
          break;
        case 4: //led_test
        ChangeDisplayTest(byte1 & 15); //calling driver 
          break;
        case 6://shutdown
        ShutdownMode( byte1 & 15); //calling driver
          break;
        case 7://led row
    setLed(0,3);
           if (firstRun == 1) {
            for (x = 0; x < 8; x++) {
            }
            firstRun = 0;
          }
          x = ((byte0 & 15) & 0x7); // mask this value so we don't write to an invalid address.
          y = byte1;

          ledmem[x] = y;
          break;
        case 8://led column
            setLed(0,4);

           if (firstRun == 1)
          {
            for (x = 0; x < 8; x++)
            {
              ledmem[x] = 0;
            }

            firstRun = 0;
          }

          x = ((byte0 & 15) & 0x7);
          y = byte1;

          for (z = 0; z < 8; z++)
          {
            if (y & (1 << z))
            {
              ledmem[z] |= 1 << x;
            }
            else
            {
              ledmem[z] &= ~(1 << x);
            }
          }
          break;
        } // end switch(address)
      } // end if (Serial.available()
         clearLed(0,1);
   
    } // end if (Serial.available();
   clearLed(0,0);
 
} // end do
  while (Serial.available() > 16);
}

int main(void)
{
	init();

	setup();
    
	for (;;)
		loop();
        
	return 0;
}

