#if DISPLAY_DRIVER_UNSPED

// NOTE by defining SPI_MAX as 1, we will activate
// the SPI version of the code that clocks the LED
// data to the max chip.  This will require that
// you also make the following pin connections
// from the arduino to the MAX7219 chip:
// Arduino pin 10 : to max 'load'
// Arduino pin 11 : to max 'din'
// Arduino pin 13 : to max 'clock'
#define SPI_MAX 0

//Max7219 (drives LEDS)
// loads in a 1 (high) or a 0 (low) on clock pulse                        
#define dataIn  4
// pulse to load bits (one bit for each clock pulse)
#define clock  6                         
// outputs all data from the IC to the LEDS
#define load  5                           

int ledstate = 0;
int ledx = 0;
int ledy = 0;
int inputrow = 0;
// change this variable to set how many MAX7219's you'll use
#define maxInUse  1

// define max7219 registers (read the tech doc for explaination)
#define max7219_reg_noop         0x00    
#define max7219_reg_digit0       0x01
#define max7219_reg_digit1       0x02
#define max7219_reg_digit2       0x03
#define max7219_reg_digit3       0x04
#define max7219_reg_digit4       0x05
#define max7219_reg_digit5       0x06
#define max7219_reg_digit6       0x07
#define max7219_reg_digit7       0x08
#define max7219_reg_decodeMode   0x09
#define max7219_reg_intensity    0x0a
#define max7219_reg_scanLimit    0x0b
#define max7219_reg_shutdown     0x0c
#define max7219_reg_displayTest  0x0f

#define DataMaskHigh  (1 << 4)
#define DataMaskLow  (~DataMaskHigh)

#define ClockMaskHigh ( 1<<6)
#define ClockMaskLow  (~ClockMaskHigh)

#define LoadMaskHigh  (1 << 5)
#define LoadMaskLow  (~LoadMaskHigh)

void displayInit()
{

   pinMode(dataPin,    OUTPUT);
  pinMode(clockPin,   OUTPUT);
  pinMode(indataPin,  INPUT);
  pinMode(inclockPin, OUTPUT);
  pinMode(inloadPin,  OUTPUT);

#if SPI_MAX
  SPCR = (1 << SPE) | (1 << MSTR);
  SPSR = (SPI2X);

  DDRB |= (1 << PB2)|(1 << PB3)|(1 << PB5);
#else
  pinMode(dataIn, OUTPUT);
  pinMode(clock,  OUTPUT);
  pinMode(load,   OUTPUT);  
#endif

  
   //initiation of the max 7219
  maxSingle(max7219_reg_scanLimit, 0x07);      
  maxSingle(max7219_reg_intensity,0x0F);
  maxSingle(max7219_reg_shutdown, 0x01);           // not in shutdown mode
  maxSingle(max7219_reg_displayTest, 0x00);        // empty registers, turn all LEDs off h

  for (int e=1; e<=8; e++){
    maxSingle(e,0);
    ledmem[e-1]=0;
  }

}

void displayWhoosh()
{
 for (int j = 0; j < 9; j++)
  {
    for (int i = 0; i < 8; i++)
    {
      maxSingle(i+1,1<<j);
    }
    delay(125);
  }
}




void putByte(byte data) {
  byte i = 8;
  byte mask;
  while(i > 0) {
    mask = 0x01 << (i - 1);      // get bitmask


    if (data & mask){            // choose bit 0 or 1
      PORTD |= DataMaskHigh; //      digitalWrite(dataIn, HIGH);// send 1
    }
    else{                        // else 
      PORTD &= DataMaskLow;   //  digitalWrite(dataIn, LOW); // send 0
    }


    PORTD &= ClockMaskLow; //    digitalWrite( clock, LOW);   // "tick" prepeare for bit input
    PORTD |= ClockMaskHigh;//    digitalWrite(clock, HIGH);   // "tock" input bit

    --i;                         // move to lesser bit
  }
}



void maxSingle( byte dig, byte seg) {    
  //maxSingle is the "easy"  function to use for a single max7219
  //dig is the row call, and seg is the column call dig and seg refer to pin names from tech doc

#if SPI_MAX

  PORTB &= ~(1 << PB2);
  SPDR = dig;
  while (!(SPSR & (1 << SPIF)));
  SPDR = seg;
  while (!(SPSR & (1 << SPIF)));
  PORTB |= (1 << PB2);

#else // SPI_MAX is 0

  PORTD &= LoadMaskLow;  //  digitalWrite(load, LOW);            // begin     

  putByte(dig);                       // specify register
  putByte(seg);                       //((data & 0x01) * 256) + data >> 1); // put data   

  PORTD |= LoadMaskHigh; //  digitalWrite(load,HIGH); 

#endif // end #if SPI_MAX
}

void ChangeIntensity(int inval)
{
  maxSingle(max7219_reg_intensity, inval & 15);
 
}

void ChangeDisplayTest(byte inval)
{
  maxSingle(max7219_reg_displayTest, inval & 15);
 
}

void ShutdownMode(byte inval)
{
   maxSingle(max7219_reg_shutdown, inval & 15); 

}

void updateLED()
{

// send the LED states to the LED matrix.
  for (int i = 0; i < 8; i++)
  {
    maxSingle(i+1,ledmem[i]);
  }
}
  
 void clearAllLights()
 {
 
     // make sure to turn off the lights.
  for (int i = 0; i < 8; i++)
  {
    maxSingle(i+1,0);
  }
 }
 
 #endif
